==> papai_unified_storage-0.6.0.tar.gz: papai_unified_storage-0.6.0/papai_unified_storage/utils.py <==
import os
from pathlib import Path
from tempfile import mkstemp
from typing import TypeVar


def create_local_dir_tree(path: str):
    """Create a directory tree in the local file system if it doesn't
    already exist.

    Parameters
    ----------
    path : str
        Path that will be created. If it is a file (i.e. it ends with a
        trailing slash), the parent directories will be created, but not the
        file itself.
    """
    path_ = Path(path)

    if not path.endswith(os.sep):
        # If the path is a file, we want to create the parent directories
        path_ = path_.parent

    path_.mkdir(parents=True, exist_ok=True)


def joinpath(*path_pieces: str | list[str]) -> str | list[str]:
    """Join paths.

    If one of the path piece is a list, the function will join each element of
    the list with the other path pieces.

    os separator in the path pieces will be remove, except for the trailing
    separator of the last piece which is used to differenciate folders from
    files.

    Parameters
    ----------
    *path_pieces : str | list[str]
        Path pieces to join.

    Returns
    -------
    str | list[str]
        the joined path: str if all the components are strings, list of strings
        otherwise.

    Examples
    --------
    >>> joinpath("a", "b", "c")
    "a/b/c"
    >>> joinpath("a", "", "c")
    "a/c"
    >>> joinpath(["a", "b"], "c/")
    ["a/c/", "b/c/"]
    >>> joinpath(["a", "b"], ["c", "d"], "e")
    ["a/c/e", "a/d/e", "b/c/e", "b/d/e"]
    >>> joinpath(["a", "", "b"], ["c", "d"], "e")
    ["a/c/e", "a/d/e", "b/c/e", "b/d/e"]
    """
    if len(path_pieces) < 2:
        raise ValueError("At least two path pieces are required.")
    if len(path_pieces) > 2:
        # Recursive call to join the first two pieces and the rest of the pieces.
        # This will slowly reduce the number of pieces until there are only two.
        return joinpath(joinpath(path_pieces[0], path_pieces[1]), *path_pieces[2:])

    first_paths = ensure_in_list(path_pieces[0])
    second_paths = ensure_in_list(path_pieces[1])

    joined_paths = _cartesian_product_join_paths(first_paths, second_paths)

    if len(joined_paths) == 1:
        return joined_paths[0]

    return joined_paths


def _cartesian_product_join_paths(
    first_path_piece: list[str], second_path_piece: list[str]
) -> list[str]:
    """Compute the cartesian product of two lists of paths.

    If one of the path piece is an empty string, it will be ignored.

    The output list will contain the concatenation of each element of the first
    list with each element of the second list.

    Returns
    -------
    list[str]
        List of length len(first_path_piece) * len(second_path_piece) that
        contains the concatenated path.
    """
    non_null_first_path_piece = [path for path in first_path_piece if path != ""]
    non_null_second_path_piece = [path for path in second_path_piece if path != ""]

    if len(non_null_first_path_piece) == 0:
        return non_null_second_path_piece
    elif len(non_null_second_path_piece) == 0:
        return non_null_first_path_piece

    results = [""] * len(non_null_first_path_piece) * len(non_null_second_path_piece)
    count = 0
    for first_path in non_null_first_path_piece:
        for second_path in non_null_second_path_piece:
            results[count] = first_path.rstrip("/") + os.sep + second_path.lstrip("/")
            count += 1

    return results[:count]


T = TypeVar("T")


def ensure_in_list(value: T | list[T]) -> list[T]:
    """Put in a list any scalar value that is not None.

    If it is already a list, do nothing.
    """
    if value is not None and not isinstance(value, list):
        value = [value]

    return value


def dummy_fn(*_, **__) -> None:
    """Dummy function that does nothing."""
    pass


def generate_temporary_filename(parent_directory: str | None = None) -> str:
    """Generate a file name in the OS temporary directory."""
    fd, file_path = mkstemp(dir=parent_directory)
    fd.close()
    return file_path

==> papai_unified_storage-0.7.0.tar.gz: papai_unified_storage-0.7.0/papai_unified_storage/utils.py <==
import os
from pathlib import Path
from tempfile import mkstemp
from typing import TypeVar

import fsspec
import fsspec.implementations


def create_dir_tree(fs: fsspec.AbstractFileSystem, path: str | list[str]):
    """Create a directory tree in the specified file system if it doesn't
    already exist.

    Parameters
    ----------
    path : str | list[str]
        Path that will be created. If it is a file (i.e. it ends with a
        trailing slash), the parent directories will be created, but not the
        file itself.
    """
    paths = ensure_in_list(path)

    for path in paths:
        path_ = Path(path)

        if not path.endswith(fs.sep):
            # If the path is a file, we want to create the parent directories
            path_ = path_.parent

        fs.makedirs(path_, exist_ok=True)


def joinpath(*path_pieces: str | list[str]) -> str | list[str]:
    """Join paths.

    If one of the path piece is a list, the function will join each element of
    the list with the other path pieces.

    os separator in the path pieces will be remove, except for the trailing
    separator of the last piece which is used to differenciate folders from
    files.

    Parameters
    ----------
    *path_pieces : str | list[str]
        Path pieces to join.

    Returns
    -------
    str | list[str]
        the joined path: str if all the components are strings, list of strings
        otherwise.

    Examples
    --------
    >>> joinpath("a", "b", "c")
    "a/b/c"
    >>> joinpath("a", "", "c")
    "a/c"
    >>> joinpath(["a", "b"], "c/")
    ["a/c/", "b/c/"]
    >>> joinpath(["a", "b"], ["c", "d"], "e")
    ["a/c/e", "a/d/e", "b/c/e", "b/d/e"]
    >>> joinpath(["a", "", "b"], ["c", "d"], "e")
    ["a/c/e", "a/d/e", "b/c/e", "b/d/e"]
    """
    if len(path_pieces) < 2:
        raise ValueError("At least two path pieces are required.")
    if len(path_pieces) > 2:
        # Recursive call to join the first two pieces and the rest of the pieces.
        # This will slowly reduce the number of pieces until there are only two.
        return joinpath(joinpath(path_pieces[0], path_pieces[1]), *path_pieces[2:])

    first_paths = ensure_in_list(path_pieces[0])
    second_paths = ensure_in_list(path_pieces[1])

    joined_paths = _cartesian_product_join_paths(first_paths, second_paths)

    if len(joined_paths) == 1:
        return joined_paths[0]

    return joined_paths


def _cartesian_product_join_paths(
    first_path_piece: list[str], second_path_piece: list[str]
) -> list[str]:
    """Compute the cartesian product of two lists of paths.

    If one of the path piece is an empty string, it will be ignored.

    The output list will contain the concatenation of each element of the first
    list with each element of the second list.

    Returns
    -------
    list[str]
        List of length len(first_path_piece) * len(second_path_piece) that
        contains the concatenated path.
    """
    non_null_first_path_piece = [path for path in first_path_piece if path != ""]
    non_null_second_path_piece = [path for path in second_path_piece if path != ""]

    if len(non_null_first_path_piece) == 0:
        return non_null_second_path_piece
    elif len(non_null_second_path_piece) == 0:
        return non_null_first_path_piece

    results = [""] * len(non_null_first_path_piece) * len(non_null_second_path_piece)
    count = 0
    for first_path in non_null_first_path_piece:
        for second_path in non_null_second_path_piece:
            results[count] = first_path.rstrip("/") + os.sep + second_path.lstrip("/")
            count += 1

    return results[:count]


T = TypeVar("T")


def ensure_in_list(value: T | list[T]) -> list[T]:
    """Put in a list any scalar value that is not None.

    If it is already a list, do nothing.
    """
    if value is not None and not isinstance(value, list):
        value = [value]

    return value


def dummy_fn(*_, **__) -> None:
    """Dummy function that does nothing."""
    pass


def generate_temporary_filename(parent_directory: str | None = None) -> str:
    """Generate a file name in the OS temporary directory."""
    fd, file_path = mkstemp(dir=parent_directory)
    fd.close()
    return file_path
